﻿//License: GPLv3 (see LICENSE.txt)

using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System;
using UnityEngine;

namespace Rotate
{
    [KSPAddon(KSPAddon.Startup.Flight, false)]
    public class VehicleRotator : MonoBehaviour
    {

        static bool started = false;
        static bool loaded = false;
        float lastrate = 1;

        static Dictionary<string, Vector3> vessel_rotations = new Dictionary<string, Vector3>();  //Rotation zu jedem Vessel

        Vessel v = FlightGlobals.ActiveVessel;

/*
        float speed = v.angularVelocity.magnitude * (kerb.seat.part.rigidbody.position - v.findWorldCenterOfMass()).magnitude;
        float force = (speed * speed) / (kerb.seat.part.rigidbody.position - v.findWorldCenterOfMass()).magnitude;
        GUILayout.BeginHorizontal();
        GUILayout.Label(kerb.name + ":");
        GUILayout.Label(force + " m/s");
 */

        private static String File
        {
            get { return KSPUtil.ApplicationRootPath + "/GameData/TimewarpRotate/Vehicle_Rotations.cfg"; }
        }
        public void CleanVesselRotations()
        {
            Dictionary<string, Vector3> temp_vessel_rotations = new Dictionary<string, Vector3>();

            foreach (KeyValuePair<string, Vector3> entry in vessel_rotations)
            {
                temp_vessel_rotations[entry.Key] = entry.Value;
            }

            vessel_rotations.Clear();

            foreach (Vessel vessel in FlightGlobals.Vessels)
            {
                vessel_rotations[vessel.id.ToString()] = Vector3.zero;
                vessel_rotations[vessel.id.ToString()] = temp_vessel_rotations[vessel.id.ToString()];
            }

            temp_vessel_rotations.Clear();
        }
        public void Save()
        {
            try
            {
                //CleanVesselRotations();

                ConfigNode save = new ConfigNode();
                //ConfigNode.CreateConfigFromObject(this, 0, save); (?)
                //ConfigNode ships = new ConfigNode("SHIPS"); (?)
                foreach (KeyValuePair<string, Vector3> entry in vessel_rotations)
                {
                    save.AddValue(entry.Key, KSPUtil.WriteVector(entry.Value));
                }
                //save.AddNode(ships); (?)

                save.Save(File);
            }
            catch (Exception e) { print("[Rotate]: An error occurred while attempting to save: " + e.Message); }
        }
        public static void Load()
        {
            loaded = true;
            ConfigNode load = ConfigNode.Load(File);
            if (load == null)
            {
                print("[Rotate]: null save");
                return;
            }
            vessel_rotations.Clear();
            //Prefill it with empty Vectors
            foreach (Vessel vprime in FlightGlobals.Vessels)
            {
                vessel_rotations[vprime.id.ToString()] = Vector3.zero;
            }
            //Fill it with the saved ones
            foreach (ConfigNode.Value s in load.values)
            {
                vessel_rotations[s.name] = KSPUtil.ParseVector3(s.value);
            }
        }


        void OnVesselGoOnRails(Vessel vprime)
        {
            if (vprime.ActionGroups[KSPActionGroup.SAS])
            {
                vessel_rotations[vprime.id.ToString()] = Vector3.zero;
            }
            else
            {
                vessel_rotations[vprime.id.ToString()] = vprime.angularVelocity;
            }

            Save();

        }
        void OnVesselGoOffRails(Vessel vprime)
        {
            v = FlightGlobals.ActiveVessel;
            
            print("[Rotate]: Vessel " + vprime.vesselName.ToString() + " now going off Rails");

            Load();

            try
            {
                Vector3 av = vessel_rotations[vprime.id.ToString()];

                Vector3 COM = vprime.findWorldCenterOfMass();
                foreach (Part p in vprime.parts) //Applying force on every part
                {
                    try
                    {
                        if (p.rigidbody == null) continue;
                        p.rigidbody.AddTorque(vprime.transform.rotation * av, ForceMode.VelocityChange);
                        p.rigidbody.AddForce(Vector3.Cross(vprime.transform.rotation * av, (p.rigidbody.position - COM)), ForceMode.VelocityChange);
                    }
                    catch (NullReferenceException nre)
                    {
                        print("Got NRE: " + nre.Message);
                    }

                }
            }
            catch (Exception e)
            {
                print("Caugt exception: " + e.Message);
                // Don't really need to do anything; we just haven't met this vehicle yet.
            }

        }

        void Start()
        {
            if (!started)
            {
                GameEvents.onVesselGoOnRails.Add(OnVesselGoOnRails);
                GameEvents.onVesselGoOffRails.Add(OnVesselGoOffRails);
                started = true;
            }

            //Prefill vessel_rotations with empty Vectors, for first time loading
            foreach (Vessel vprime in FlightGlobals.Vessels)
            {
               vessel_rotations[vprime.id.ToString()] = Vector3.zero;
            }
        }

        public void RotateWhilePacked(Vessel vessel)
        {
            bool rotate = true;
            Vector3 angvel = vessel_rotations[vessel.id.ToString()];

            if (vessel.packed == true && vessel.situation != Vessel.Situations.LANDED
                                      && vessel.situation != Vessel.Situations.PRELAUNCH
                                      && vessel.situation != Vessel.Situations.DOCKED
                                      && vessel.situation != Vessel.Situations.SPLASHED)
            {
                if (vessel == FlightGlobals.ActiveVessel)
                {
                    if (vessel.Autopilot.Enabled && vessel.Autopilot.Mode == VesselAutopilot.AutopilotMode.StabilityAssist)
                    {
                        rotate = false;
                    }
                }

                if (rotate == true)
                {
                    float angle = angvel.magnitude * TimeWarp.CurrentRate;
                    Quaternion rotation = Quaternion.AngleAxis(angle, angvel);
                    vessel.SetRotation(vessel.GetTransform().rotation * rotation);
                }

                //TODO: SAS Modes
            }
        }

        void FixedUpdate()
        {
            if (!loaded)
            {
                Load();
            }

            try
            {
                foreach (Vessel vessel in FlightGlobals.Vessels)
                {
                    RotateWhilePacked(vessel);
                }
                lastrate = TimeWarp.CurrentRate;
            }
            catch (NullReferenceException nre)
            {
                print("[Rotate]: NullReference: " + nre.ToString());
            }
        }
    }
}

